<?php
require_once('Admin.inc.php');

final class AdminT12 extends Admin {
	private $tab;

	/**
	 * Constructor
	 *
	 * @param string $login
	 * @param string $pass
	 * @param boolean $strict_mode  check form data strict or messily
	 * @param int $tab  show tab==1 or tab==2, (choice between unconfirmed and confirmed publications list)
	 */
	public function __construct($post, $get, $login, $pass, $strict_mode, $tab) {
		parent::__construct($post, $get, $login, $pass, $strict_mode);
		$this->tab = $tab;
	}

	/**
	 * Execute admin tab1 or tab2
	 *
	 */
	public function execute() {
		 if ($this->post['delete_files'] && $this->get['_publication_id']) {
			try {
				$this->delete_files($this->get['_publication_id']);
				$this->smarty->assign('operation_success', 'Soubory byly smazány.');
				$this->display_publ_list_short($this->get['_publication_id']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_publ_list_short($this->get['_publication_id']);
			}
		} elseif ($this->post['add_file'] && $this->get['_publication_id']) {
			try {
				$this->add_file($this->get['_publication_id']);
				$this->smarty->assign('operation_success', 'Soubory aktualizovány.');
				$this->display_publ_list_short($this->get['_publication_id']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_publ_list_short($this->get['_publication_id']);
			}
		} elseif ($this->post['confirm'] && $this->get['_publication_id']) {
			try {
				$this->confirm($this->get['_publication_id'], $this->post['propagated_attributes']);
				$this->smarty->assign('operation_success', 'Úspěšně schváleno.');
				$this->smarty->assign('refresh', "1 ; URL=index.php");
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['unconfirm'] && $this->get['_publication_id']) {
			try {
				$this->confirm($this->get['_publication_id'], $this->post['propagated_attributes'], 0); //unconfirm
				$this->smarty->assign('operation_success', 'Publikace byla přesunuta do neschválených.');
				$this->smarty->assign('refresh', "1 ; URL=index.php?tab=2");
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['deny'] && $this->get['_publication_id']) {
			try {
				$this->delete($this->get['_publication_id']);
				$this->smarty->assign('operation_success', 'Publikace smzána.');
				$this->smarty->assign('refresh', "1 ; URL=index.php?tab=".$this->tab);
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['modify'] && $this->get['_publication_id']) {
			try {
				$this->display_modify_publication($this->get['_publication_id']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['modify_form'] && $this->get['_publication_id']) {
			try {
				if ($this->post['save']) { //save changes
					$this->validate_modify_publication();
					$this->update_publication($this->get['_publication_id']);
					$this->clear_temporaries();
					$this->smarty->assign('operation_success', 'Publikace změněna.');
					$this->smarty->assign('refresh', "1 ; URL=index.php?tab=".$this->get['tab']."&amp;_publication_id=".$this->get['_publication_id']);
					$this->smarty->display('admin_tab12.tpl');
				} elseif ($this->post['add_attribute']) { //add temporary attribute
					$this->validate_add_att();
					$this->add_tmp_attribute($this->post['new_attribute_name'], $this->post['new_attribute_description'], $this->post['new_attribute_value']);
					$this->smarty->assign('operation_success', 'Atribut bude po uložení změn přidán');
					$this->display_modify_publication($this->get['_publication_id']);
				} elseif ($this->post['mark_attributes_deleted']) {  //unset all temporary attributes, mark real attributes as deleted
					$_SESSION['feedback_modify'] = $this->post;
					$_SESSION['attributes_deleted'] = 1;
					unset($_SESSION['tmp_attr']);
					$this->smarty->assign('operation_success', 'Po uložení změn budou neveřejné atributy smazány');
					$this->display_modify_publication($this->get['_publication_id']);
				}
			} catch(SAException $s) {
				die($s->getMessage().'<br/>Line:'.$s->getLine());
			} catch (Exception $e) {
				$this->feedback_modify();
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_modify_publication($this->get['_publication_id']);
			}
		} elseif ($this->get['_publication_id']) {
			$this->clear_temporaries();
			$this->display_publ_list_short($this->get['_publication_id']);
		} else {
			$this->clear_temporaries();
			$this->display_publ_list();
		}
	}

	/**
	 * Display modify publication form
	 *
	 * @param int $_publication_id  publication_id
	 */
	private function display_modify_publication($_publication_id) {
		$submitter_id = $this->db->getOne("SELECT submitter_id FROM `publication` WHERE id='$_publication_id'");
		$sql = "SELECT id, CONCAT_WS(' ',name,middlename,surname) AS author
				FROM `author`
				WHERE confirmed = '1' OR submitter_id = '".$submitter_id."'
				ORDER BY surname";
		$this->smarty->assign('authors', htmlspecialchars_rec($this->db->getAssoc($sql)));
		$sql = "SELECT id, name
				FROM `publisher`
				WHERE confirmed='1' OR submitter_id = '".$submitter_id."'
				ORDER BY name";
		$this->smarty->assign('publisher', htmlspecialchars_rec($this->db->getAssoc($sql)));
		$sql = "SELECT id, name
				FROM `categories`
				WHERE confirmed='1' OR submitter_id = '$submitter_id'
				ORDER BY name";
		$this->smarty->assign('categories', htmlspecialchars_rec($this->db->getAssoc($sql)));
		//take confirmed results, or results owned by current user
		$this->smarty->assign('strict_mode', $this->strict_mode);
		$this->smarty->assign('tab', $this->get['tab']);
		$this->smarty->assign('_publication_id', $_publication_id);
		$this->smarty->assign('modify', 1); //decide short_look/modify part of the template
		$this->smarty->assign("type_no", array(1, 2, 3));
		$this->smarty->assign("type_names", array("Kniha", "Časopis", "Konference"));
		$this->smarty->assign("lang_no", array(1,2,3));
		$this->smarty->assign("langs", array("Czech", "English", "German"));
		$this->smarty->assign("country", countries());
		$this->smarty->assign("state", states());
		if (!$_SESSION['feedback_modify']) {
			//get from db and display
			$res =& $this->db->query("SELECT * FROM `publication` WHERE id='$_publication_id'");
			if(PEAR::isError($res))
			throw new SAException($res->getMessage());
			if(!$res->numRows()) {
				$this->smarty->assign('operation_error', 'Publikace neexistuje.');
				$this->smarty->display('admin_tab12.tpl');
			}
			$row =& $res->fetchRow(DB_FETCHMODE_ASSOC);
			$author_ids =& $this->db->getAssoc("SELECT DISTINCT id, id FROM `author` JOIN `author_has_publication` ON author.id=author_has_publication.author_id WHERE publication_id='$_publication_id'");
			if (PEAR::isError($author_ids))
				throw new SAException($author_ids->getMessage());
			$row = htmlspecialchars_rec($row);
			$this->smarty->assign('decision_pub_type', $row['pub_type']);
			$this->smarty->assign('selected_pub_type', $row['pub_type']);
			$this->smarty->assign('selected_pub_lang', $row['language']);
			$this->smarty->assign('selected_country', $row['country_code']);
			$this->smarty->assign('selected_state', $row['state_code']);
			$this->smarty->assign('pub_title', $row['title']);
			$this->smarty->assign('city', $row['city']);
			$this->smarty->assign('selected_authors', $author_ids);
			$this->smarty->assign('selected_publisher', $row['publisher_id']);
			$sql = "SELECT categories_id
					FROM `categories_has_publication`
					WHERE publication_id = '$_publication_id'";
			$selected_categories =& $this->db->getAll($sql, null, DB_FERCHMODE_ORDERED);
			if (PEAR::isError($selected_categories))
				throw new SAException($selected_categories->getMessage());
			foreach ($selected_categories as $i => $v) {
				$selected_categories[$i] = $v[0];
			}
			$this->smarty->assign('selected_categories', $selected_categories);
			$this->smarty->assign('isbn', $row['isbn']);
			$this->smarty->assign('pp', $row['pp']);
			$this->smarty->assign('issue_date', $row['issue_date']);
			$this->smarty->assign('vol', $row['vol']);
			$this->smarty->assign('no', $row['no']);
			$this->smarty->assign('w_from', $row['w_from']);
			$this->smarty->assign('w_to', $row['w_to']);
			$this->smarty->assign('workshop_name', $row['workshop_name']);
		} else
			$this->feedback_modify();
		//--------Atttibutes follow---------
		//public attributes
		$sql = "SELECT id, name FROM attributes WHERE confirmed = '1'";
		$arr =& $this->db->getAssoc($sql);
		$sql = "SELECT id, description FROM attributes WHERE confirmed = '1'";
		$desc =& $this->db->getAssoc($sql);
		$s='';
		foreach ($arr as $i => $v) {
			$value = $this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$i."' AND publication_id='$_publication_id'");
			$s .= '<tr><td><strong>'.$v.'</strong>'.($desc[$i]?'<br />('.htmlspecialchars($desc[$i]).')':'').'</td><td colspan="2"><input type="text" name="'.$v.'" size="80" value="'.htmlspecialchars($value).'" /></td></tr>';
		}
		//unpublished attributes
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attrib_storage` JOIN `attributes`
				ON attrib_storage.attributes_id=attributes.id
				WHERE attrib_storage.publication_id = '$_publication_id' and confirmed=0";
		$all = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$n='';
		if (is_array($all)) foreach ($all as $arr) {
			if (!$_SESSION['attributes_deleted'])
				$n .= '<tr><td colspan="3"><strong>'.$arr['name'].':</strong> <cite>'.
				htmlspecialchars($this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$arr['id']."' AND publication_id='$_publication_id'")).'</cite>'.
				'<br />('.htmlspecialchars($arr['description']).')</td></tr>';
		}
		//temporarily added new attributes
		$t='';
		$tmp = $_SESSION['tmp_attr']['name']; //$tmp is an array
		$tmp = stripslashes_rec(htmlspecialchars_rec($tmp));
		if (is_array($tmp)) {
			foreach ($tmp as $i => $value) {
				$description = stripslashes(htmlspecialchars($_SESSION['tmp_attr']['description'][$i]));
				$t .= '<tr><td><strong>'.$i.'</strong>: <cite>'.$value.'</cite>'.($description?'<br />('.$description.')':'').'</td></tr>';
			}
		}
		$this->smarty->assign('tmp_attr', $n.$t);
		$this->smarty->assign('attributes', $s);
		$this->smarty->display('admin_tab12.tpl');
	}

	/**
	 * Validate form data sent through modify form
	 *
	 */
	private function validate_modify_publication() {
		$_SESSION['feedback_modify'] = $this->post; //save typed form data
		extract($this->post);
		if (!($pub_type==1 || $pub_type==2 || $pub_type==3))
			throw new Exception('Není vybrán typ publikace');
		if (!($pub_lang==1 || $pub_lang==2 || $pub_lang==3))
			throw new Exception('Není vybrán jazyk');
		if (!$pub_title)
			throw new Exception('Chybí název publikace');
		if (!is_array($authors))
			throw new Exception('Nejsou vybráni autoři');
		if ($this->strict_mode) {
			if (!$city)
				throw new Exception('Chybí město');
			if (!($country && countries($country)))
				throw new Exception('Není vybrána země');
			if (($country=='US' || $country=='CA') && !($state && ($state?states($state):false)))
				throw new Exception('Není vybrán stát');
			if ($country!='US' && $country!='CA')
				unset($_SESSION['feedback_modify']['state']);
			switch ($_SESSION['feedback_modify']['pub_type']) {
				case(1): //Book
					if (!$isbn)
						throw new Exception('Chybí ISBN');
					break;
				case(2): //magazine
					if (!$issn)
						throw new Exception('Chybí ISSN');
					if (!$vol || !is_numeric($vol))
						throw new Exception('Chybí vol. časopisu');
					if (!$no || !is_numeric($no))
						throw new Exception('Chybí no. časopisu');
					if (!$pp)
						throw new Exception('Rozsah stránek je chybí');
					break;
				case(3): //workshop
					if (!$isbn)
						throw new Exception('Chybí ISBN/ISSN');
					if (!$workshop_name)
						throw new Exception('Chybí jméno konference');
					if (!$w_from)
						throw new Exception('Chybí datum zahájení konference, zadejte YYYY-MM-DD', 1);
					if (!$w_to)
						throw new Exception('Chybí datum ukončení konference, zadejte YYYY-MM-DD', 1);
					break;
				default:
					throw new SAException('Interní chyba, neplatný typ publikace');
			}
		}
	}

	/**
	 * Validate temporarily added extensible attributes
	 *
	 */
	private function validate_add_att(){
		$_SESSION['feedback_modify'] = $this->post; //save typed form data
		extract($this->post);
		if (!ereg('^_([0-9]|[a-zA-Z]){0,31}$', $new_attribute_name))
			throw new Exception('První znak jména vkládaného atributu musí být podtržítko, jméno může obsahovat písmena anglické abecedy a číslice, max. však 32 znaků.');
		if (!$new_attribute_value)
			throw new Exception('Chybí hodnota pro nově vkládaný atributut');
		if (sizeof($new_attribute_description) > 255)
			throw new Exception('Popisek nového atributu je příliš dlouhý');
		unset($_SESSION['feedback_modify']['new_attribute_name']);
		unset($_SESSION['feedback_modify']['new_attribute_description']);
		unset($_SESSION['feedback_modify']['new_attribute_value']);
	}

	/**
	 * Gives written data back to form modify form
	 *
	 */
	private function feedback_modify() {
		$feedback = $_SESSION['feedback_modify'];
		$this->smarty->assign('selected_pub_type', $feedback['pub_type']);
		$this->smarty->assign('selected_pub_lang', $feedback['pub_lang']);
		$this->smarty->assign('selected_country', $feedback['country']);
		$this->smarty->assign('selected_state', $feedback['state']);
		$this->smarty->assign('pub_title', $feedback['pub_title']);
		$this->smarty->assign('city', $feedback['city']);
		$this->smarty->assign('selected_authors', $feedback['authors']);
		$this->smarty->assign('selected_publisher', $feedback['publisher']);
		$this->smarty->assign('selected_categories', $feedback['propagated_categories']);
        $this->smarty->assign('isbn', $feedback['isbn']);
        $this->smarty->assign('issn', $feedback['issn']);
        $this->smarty->assign('pp', $feedback['pp']);
        $this->smarty->assign('issue_date', $feedback['issue_date']);
        $this->smarty->assign('vol', $feedback['vol']);
        $this->smarty->assign('no', $feedback['no']);
        $this->smarty->assign('w_from', $feedback['w_from']);
        $this->smarty->assign('w_to', $feedback['w_to']);
		$this->smarty->assign('workshop_name', $feedback['workshop_name']);
		$this->smarty->assign('new_attribute_name', $feedback['new_attribute_name']);
		$this->smarty->assign('new_attribute_value', $feedback['new_attribute_value']);
		$this->smarty->assign('new_attribute_description', $feedback['new_attribute_description']);
	}


	/**
	 * General purpose = when admin goes back from "modify puplication section"
	 * in order to discard his changes.
	 */
	private function clear_temporaries() {
		unset($_SESSION['feedback_modify']);
		unset($_SESSION['tmp_attr']);
		unset($_SESSION['attributes_deleted']);
	}

	/**
	 * Confirm publication
	 *
	 * @param int $publication_id
	 * @param array of int $propagated_attributes  list of ids of extra attributes,
	 * which are supposed to be public for all. Format must be: array(0 => attributes_id, 1 => arrtibutes_id, ...)
	 * @param  use 1 to confirm, 0 to unconfirm the publication
	 */
	private function confirm($publication_id, $propagated_attributes=null, $confirmed=1) {
		$this->db->autoCommit(false);
		//update `publication`
		$res =& $this->db->query("UPDATE `publication` SET confirmed='$confirmed' WHERE id='$publication_id'");
		if(DB::isError($res)) {
			$this->db->rollback();
			throw new SAException($res->getMessage());
		}
		//update `author`
		if ($confirmed==1) {//do this only if confirming, backward process is impossible
			$author_ids = $this->db->getAll("SELECT DISTINCT id FROM `author` JOIN `author_has_publication` ON author.id=author_has_publication.author_id WHERE publication_id='$publication_id'", array(), DB_FETCHMODE_ORDERED);
			if(DB::isError($author_ids)) {
				$this->db->rollback();
				throw new SAException($author_ids->getMessage());
			}
			foreach ($author_ids as $aid) {
				$aid = $aid[0];
				$res =& $this->db->query("UPDATE `author` SET confirmed=1 WHERE confirmed=0 AND id='$aid'");
				if(DB::isError($res)) {
					$this->db->rollback();
					throw new SAException($res->getMessage());
				}
			}
		}
		//update `publisher`
		$publisher_id =& $this->db->getOne("SELECT publisher_id FROM `publication` WHERE id='$publication_id'");
		$res =& $this->db->query("UPDATE `publisher` SET confirmed='$confirmed' WHERE id='$publisher_id' AND confirmed=0");
		if(DB::isError($res)) {
			$this->db->rollback();
			throw new SAException($res->getMessage());
		}
		//update `categories` - confirm ALL recently added categories
		if ($confirmed==1) {//do this only if confirming, backward process is impossible
			$sql = "SELECT categories_id
					FROM `categories_has_publication` JOIN `categories`
					ON categories_has_publication.categories_id=categories.id
					WHERE publication_id='$publication_id' AND categories.confirmed=0";
			$cat_ids =& $this->db->getAll($sql); //take all unconfirmed categories that depends on the current publication
			if(DB::isError($cat_ids)) {
				$this->db->rollback();
				throw new SAException($cat_ids->getMessage());
			}
			$sth =& $this->db->prepare("UPDATE `categories` SET confirmed=1 WHERE id=?");
			if(DB::isError($sth)) {
				$this->db->rollback();
				throw new SAException($sth->getMessage());
			}
			foreach($cat_ids as $v) {
				$id = $v[0]; //0 - take first row
				$res =& $this->db->execute($sth, $id);
				if(DB::isError($res)) {
					$this->db->rollback();
					throw new SAException($res->getMessage());
				}
			}
		}
		//update `attributes` - take all unconfirmed attributes dependnt to the current
		//publication and if there are some, compare their ids to the $propagated_attributes
		if ($confirmed==1) {//do this only if confirming, backward process is impossible
			$sql = "SELECT DISTINCT id
					FROM `attrib_storage` JOIN `attributes`
					ON attrib_storage.attributes_id=attributes.id
					WHERE confirmed=0 AND attrib_storage.publication_id = '$publication_id'";
			$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ORDERED);
			if (is_array($attributes)) foreach ($attributes as $id) {
				$id = $id[0];
				if (in_array($id, $propagated_attributes)) {//decide whether administrator wants this attribute to be public or not
					$res =& $this->db->query("UPDATE `attributes` SET confirmed=1 WHERE id='$id' AND confirmed=0");
					if (PEAR::isError($res)) {
						$this->db->rollback();
			    		throw new SAException($res->getMessage());
					}
				}
			}
		}
		$this->db->commit();
	}

	/**
	 * Delete publication
	 *
	 * @param int $publication_id
	 */
	private function delete($publication_id) {
		$this->db->autoCommit(false);
		$sql = "SELECT categories_id, publication_id, confirmed
				FROM `categories_has_publication` JOIN `categories`
				ON categories_has_publication.categories_id=categories.id
				GROUP BY categories_id
				HAVING COUNT(categories_id) < 2
				AND confirmed=0
				AND publication_id='$publication_id'";
		$should_del =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC); //take all unconfirmed categories that depends on the current publication (and are refferencen by only one publication)
		if(DB::isError($should_del)) {
			$this->db->rollback();
			throw new SAException($should_del->getMessage());
		}
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		//delete from `categories_has_publication`
		$res =& $this->db->query("DELETE FROM `categories_has_publication` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}*/
		//delete from `categories` - delete ALL unconfirmed, previously added categories
		$sth =& $this->db->prepare("DELETE FROM `categories` WHERE id=?");
		if(DB::isError($sth)) {
			$this->db->rollback();
			throw new SAException($sth->getMessage());
		}
		foreach($should_del as $v) {
			$id = $v['categories_id']; //0 - take first row
			$res =& $this->db->execute($sth, $id);
			if(DB::isError($res)) {
				$this->db->rollback();
				throw new SAException($res->getMessage());
			}
		}
		//---
		$sql = "SELECT publication_id, attributes_id
				FROM `attrib_storage`
				GROUP BY attributes_id
				HAVING COUNT(attributes_id) < 2 AND publication_id='$publication_id'";
		$relevant =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if(PEAR::isError($relevant)) {
			$this->db->rollBack();
			throw new SAException($relevant->getMessage());
		}
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		//delete form `attrib_storage`
		$res =& $this->db->query("DELETE FROM `attrib_storage` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}*/
		//delete from `attributes`
		if (is_array($relevant)) foreach($relevant as $row) {//delete relevant attributes (refferenced only once)
			$this->db->query("DELETE FROM `attributes` WHERE confirmed=0 AND id='".$row['attributes_id']."'");
		}
		//delete from `author`
		$sql = "SELECT publication_id, author_id
				FROM `author_has_publication`
				GROUP BY author_id
				HAVING COUNT(author_id) < 2 AND publication_id='$publication_id'";
		$relevant =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if(PEAR::isError($relevant)) {
			$this->db->rollBack();
			throw new SAException($relevant->getMessage());
		}
		if (is_array($relevant)) foreach($relevant as $row) {//delete relevant authors (refferenced only once)
			$this->db->query("DELETE FROM `author` WHERE confirmed=0 AND id='".$row['author_id']."'");
		}
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		//delete form `author_has_publication`
		$res =& $this->db->query("DELETE FROM `author_has_publication` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}*/

		//delete from `publisher`
		/* Here MySQL 4.1 fails, bug ?
		$sql = "DELETE
				FROM `publisher`
				WHERE confirmed=0 AND id =
				(SELECT publisher_id
				FROM `publication`
				GROUP BY publisher_id
				HAVING COUNT(id) < 2 AND publisher_id =
				(SELECT publisher_id
				FROM publication JOIN publisher ON publication.publisher_id=publisher.id
				WHERE publication.id='$publication_id'))";
		------------------------------------------------- it's solved with following code*/
		$sql = "SELECT publisher_id
				FROM `publication`
				GROUP BY publisher_id
				HAVING COUNT(id) < 2 AND publisher_id =
				(SELECT publisher_id
				FROM publication JOIN publisher ON publication.publisher_id=publisher.id
				WHERE publication.id='$publication_id')";
		$publisher_id =& $this->db->getOne($sql); //delete relevant publishers (refferenced only once)
		if(PEAR::isError($publisher_id)) {
			$this->db->rollBack();
			throw new SAException($publisher_id->getMessage());
		}
		$res =& $this->db->query("DELETE FROM `publisher` WHERE confirmed=0 AND id='$publisher_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		//delete from `publication`
		$res =& $this->db->query("DELETE FROM `publication` WHERE id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		$this->delete_files($publication_id);
		$this->db->commit();
	}

	/**
	 * Creates a temporary information about new attributes
	 *
	 * @param string $name  name of the new added attribute
	 * @param string $description  description of the new added attribute
	 * @param string $value  text value of the attribute
	 */
	private function add_tmp_attribute($name, $description, $value) {
		$_SESSION['tmp_attr']['name'][$name] = $value;
		$_SESSION['tmp_attr']['description'][$name] = $description;
	}


	/**
	 * Display tab 1 or 2 entry point (show list of unconfirmed or list of confirmed publications)
	 *
	 */
	public function display_publ_list() {
		if ($this->tab == 1) //display unconfirmed publications
			$res =& $this->db->query("SELECT id, title, submitter_id FROM `publication` WHERE confirmed=0 OR confirmed=NULL");
		elseif ($this->tab == 2) //display confirmed
			$res =& $this->db->query("SELECT id, title, submitter_id FROM `publication` WHERE confirmed=1");
		$publications = '';
		while ($row =& $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$submitter = $this->db->getOne("SELECT ctu_login FROM `submitter` WHERE id='".$row['submitter_id']."'");
			$publications .= '<a href="index.php?tab='.$this->tab.'&amp;_publication_id='.$row['id'].'">'.htmlspecialchars($row['title']).'</a><br />';
		}
		$this->smarty->assign('tab', $this->tab);
		$this->smarty->assign('publications', $publications);
		$this->smarty->display('admin_tab12.tpl');
	}


	/**
	 * Display publications details list and allow/deny form
	 *
	 * @param int $_publication_id  publication_id
	 */
	private function display_publ_list_short($_publication_id) {
		$res_publication =& $this->db->query("SELECT * FROM `publication` JOIN `publisher` ON publication.publisher_id=publisher.id WHERE publication.id='$_publication_id'");
		if (!$res_publication->numRows()) $res_publication =& $this->db->query("SELECT * FROM `publication` WHERE publication.id='$_publication_id'");
		if (PEAR::isError($res_publication))
			throw new SAException($res_publication->getMessage());
		$pub =& $res_publication->fetchRow(DB_FETCHMODE_ASSOC);
		$author_names =& $this->db->getAll("SELECT DISTINCT CONCAT_WS(' ', name, middlename, surname) FROM `author` JOIN `author_has_publication` ON author.id=author_has_publication.author_id WHERE publication_id='$_publication_id'", array(), DB_FETCHMODE_ORDERED);
		if (PEAR::isError($author_names))
			throw new SAException($author_names->getMessage());
		$count = count($author_names); $i = 0; $authors_tog = '';
		foreach ($author_names as $a_name) {
			if (++$i < $count) $authors_tog .= $a_name[0].', ';
			else $authors_tog .= $a_name[0];
		}
		$pub = htmlspecialchars_rec($pub);
		extract($pub);
		$date_arr = explode('-', $issue_date);
		$this->smarty->assign('_publication_id', $_publication_id);
		$this->smarty->assign('short_look', 1);  //decide short_look/modify part of the template
		$this->smarty->assign('pub_type', $pub_type);
		$this->smarty->assign('pub_lang', $language);
		$this->smarty->assign('country', $country_code?countries($country_code):'');
		$this->smarty->assign('state', ($state_code?states($state_code):''));
		$this->smarty->assign('pub_title', $title);
		$this->smarty->assign('city', $city);
		$this->smarty->assign('authors', htmlspecialchars($authors_tog)); //associative authors
		$this->smarty->assign('publisher', $name); //publisher name
		$this->smarty->assign('isbn', $isbn);
		$this->smarty->assign('issn', $isbn);
		$this->smarty->assign('pp', $pp);
		$this->smarty->assign('issue_date', (($pub_type==1 || $pub_type==3)?$date_arr[0]:$date_arr[0].'-'.$date_arr[1]));
		$this->smarty->assign('vol', $vol);
		$this->smarty->assign('no', $no);
		$this->smarty->assign('workshop_name', $workshop_name);
		$this->smarty->assign('execution_date', $w_from=='0000-00-00'?'':$w_from." -> ".$w_to);
		//categories
		$sql = "SELECT id, name, confirmed
				FROM `categories_has_publication` JOIN `categories`
				ON categories_has_publication.categories_id = categories.id
				WHERE categories_has_publication.publication_id = '$_publication_id'";
		$categories =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($categories)) {
			throw new SAException($categories->getMessage());
		}
		$this->smarty->assign('categories', $categories);
		//attributes
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attrib_storage` JOIN `attributes`
				ON attrib_storage.attributes_id=attributes.id
				WHERE attrib_storage.publication_id = '$_publication_id'";
		$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($attributes)) {
			throw new Exception($attributes->getMessage());
		}
		$s = '';
		if (is_array($attributes))
		foreach ($attributes as $arr) {
			$s .=
			'<tr>
				<td><input type="checkbox" name="propagated_attributes[]" value="'.$arr['id'].'" '.($arr['confirmed']?'checked="checked" disabled="disabled"':'').' /></td>
				<td><strong>'.$arr['name'].'</strong></td>'.'
				<td>'.htmlspecialchars($arr['description']).'</td>'.'
				<td>'.htmlspecialchars($this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$arr['id']."' AND publication_id='$_publication_id'")).'</td>
			</tr>';
		}
		$this->smarty->assign('attributes', $s);
		$this->smarty->assign('attached_files', $this->file_list($_publication_id));
		$this->smarty->assign('tab', $this->tab);
		$this->smarty->display('admin_tab12.tpl');
	}

	/**
	 * Return the file list attached to a publication
	 *
	 * @param int  $publication_id
	 * @return array  array of string with file names
	 */
	private function file_list($publication_id) {
		$path="../storage/$publication_id";
		if(file_exists($path)) {
			$handle=opendir($path);
			while ($file = readdir($handle)) {
				if (is_file($path.'/'.$file)){
					$tab[]=$file;
				}
			}
			return $tab;
		}
	}

	/**
	 * Delete file from its storage location
	 *
	 * @param int $_publication_id
	 */
	private function delete_files($publication_id) {
		$handle=@opendir("../storage/$publication_id/");
		if (!$handle) return;
		while ($file = readdir($handle)) {
			if (is_file($path="../storage/$publication_id/".$file)){
				unlink($path);
			}
		}
	}

	/**
	 * Add file
	 *
	 * @param int  $publication_id
	 */
	private function add_file($publication_id) {
		$file =& $_FILES['file']['tmp_name'];
		$name =& $_FILES['file']['name'];
		$extension = substr($name, strrpos($name, ".") + 1);
		if ((!is_uploaded_file($file) && $file))
			throw new SAException("Not an uploaded file!");
		if ($file) {
			if(!file_exists("../storage/$publication_id"))
				mkdir("../storage/$publication_id");
			$res = move_uploaded_file($file, "../storage/$publication_id/".$name);
			if (!$res) {
				throw new SAException('can not move uploaded file .pdf');
			}
		}
	}

	/**
	 * Update publication
	 *
	 * @param int $_publication_id  publication id
	 */
	protected function update_publication($_publication_id) {
		$submitter_id = $this->auth_user_id;
		$data = $_SESSION['feedback_modify'];  //pull saved data (saved by validate_modify_publication())
		$publisher_id = $data['publisher'];
		$country_code = $data['country'];
		$state_code = $data['state'];
		$city = $data['city'];
		$pub_type = $data['pub_type'];
		$language = $data['pub_lang'];
		$title = $data['pub_title'];
		$authors = $data['authors']; //an array
		$isbn = $data['isbn'];
		if ($pub_type == 1) {
			$issue_date = $data['issue_date'].'-01';
			$pp = $data['pp'];
		} elseif ($pub_type == 2) {
			$issue_date = $data['issue_date'].'-01';
			$vol = $data['vol'];
			$no = $data['no'];
			$pp = $data['pp'];
		} elseif ($pub_type == 3) {
			$issue_date = $data['issue_date'].'-01';
			$workshop_name = $data['workshop_name'];
			$w_from = $data['w_from'];
			$w_to = $data['w_to'];
			$pp = $data['pp'];
		}
		$this->db->autoCommit(false);
		//update `publication`
		$sql = "UPDATE `publication` SET
				publisher_id='$publisher_id',
				country_code='$country_code',
				state_code='$state_code',
				city='$city',
				pub_type='$pub_type',
				language='$language',
				title='$title',
				issue_date='$issue_date',
				w_from='$w_from',
				w_to='$w_to',
				workshop_name='$workshop_name',
				isbn='$isbn',
				vol='$vol',
				no='$no',
				pp='$pp'
				WHERE id='$_publication_id'";
		$res =& $this->db->query($sql);
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		//delete old rows and insert new rows into `author_has_publication`
		$res =& $this->db->query("DELETE FROM `author_has_publication` WHERE publication_id='$_publication_id'");
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		$sth = $this->db->prepare("INSERT INTO `author_has_publication` VALUES (?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SAException($sth->getMessage());
		}
		if (is_array($authors)) foreach ($authors as $_id) {
			$res =& $this->db->execute($sth, array($_id, $_publication_id));
			if (PEAR::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
		}
		//insert/delete from `categories_has_publication`
		$propagated_categories = $this->post['propagated_categories'];//what has been checked in the form
		if(!$propagated_categories) $propagated_categories = array();
		$assigned_categories = $this->assigned_categories($_publication_id);//what is in database
		if(!$assigned_categories) $assigned_categories = array();
		if (is_array($propagated_categories)) {
			$sth =& $this->db->prepare("INSERT INTO `categories_has_publication` VALUES(?, ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SAException($sth->getMessage());
			}
			foreach ($propagated_categories as $category_id) {
				if(!in_array($category_id, $assigned_categories)) {
					$res =& $this->db->execute($sth, array($_publication_id, $category_id));
					if (DB::isError($res)) {
						$this->db->rollBack();
						throw new SAException($res->getMessage());
					}
				}

			}
		}
		$unwanted_categories = array_diff($assigned_categories, $propagated_categories);
		if (is_array($unwanted_categories)) {
			$sth =& $this->db->prepare("DELETE FROM `categories_has_publication` WHERE categories_id=? AND publication_id='$_publication_id'");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SAException($sth->getMessage());
			}
			$res =& $this->db->executeMultiple($sth, $unwanted_categories);//delete unwanted category bundles form table `categories_has_publication`
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
		}
		//----
		if ($_SESSION['attributes_deleted']) {
			$sql = "SELECT publication_id, attributes_id
					FROM `attrib_storage`
					GROUP BY attributes_id
					HAVING COUNT(attributes_id) < 2 AND publication_id='$_publication_id'";
			$relevant =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
			if(PEAR::isError($relevant)) {
				$this->db->rollBack();
				throw new SAException($relevant->getMessage());
			}
			//delete from `attrib_storage` - only private(unconfirmed) attributes
			$res =& $this->db->query("SELECT attributes_id as id FROM `attrib_storage`
									WHERE publication_id='$_publication_id' AND attributes_id IN
									(SELECT attributes_id
									FROM `attrib_storage` JOIN `attributes`
									ON attrib_storage.attributes_id=attributes.id
									WHERE confirmed=0 AND publication_id='$_publication_id')");
			if(PEAR::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
			while ($row =& $res->fetchRow()) {
				$rd =& $this->db->query("DELETE FROM `attrib_storage` WHERE publication_id='$_publication_id' AND attributes_id=".$row[0]);
				if(PEAR::isError($rd)) {
					$this->db->rollBack();
					throw new SAException($rd->getMessage());
				}
			}
			//delete from `attributes` if private and relevant (refferenced only once)
			if (is_array($relevant)) foreach($relevant as $row) {
				$res =& $this->db->query("DELETE FROM `attributes` WHERE confirmed=0 AND id='".$row['attributes_id']."'");
				if(PEAR::isError($res)) {
					$this->db->rollBack();
					throw new SAException($res->getMessage());
				}
			}
		}
		//insert new attributes added by admin
		$attributes = $_SESSION['tmp_attr']['name'];
		if(is_array($attributes)) foreach ($attributes as $name => $value) {
			$description = $_SESSION['tmp_attr']['description'][$name]; //attribute description
			//insert into `attributes
			$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', '0')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
			$attributes_id = last_insert_id($this->db);
			//insert into `attrib_storage`
			$sql = "INSERT INTO `attrib_storage` VALUES ('$_publication_id', '$attributes_id', '$submitter_id', '$value')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
		}
		$public_attr = $this->filled_ext_attr();
		if (is_array($public_attr)) { //prepare attribute insertion
			$sth1 = $this->db->prepare("INSERT INTO `attrib_storage` VALUES ('$_publication_id', ?, '$submitter_id', ?)");
			$sth2 = $this->db->prepare("UPDATE `attrib_storage` SET value=?
										WHERE publication_id='$_publication_id' AND attributes_id=?");
			if (PEAR::isError($sth1)) {
				$this->db->rollBack();
				throw new SAException($sth1->getMessage());
			}
			if (PEAR::isError($sth2)) {
				$this->db->rollBack();
				throw new SAException($sth2->getMessage());
			}
			foreach ($public_attr as $id => $name) {
				//insert into `attrib_storage`
				$nr =& $this->db->query("SELECT * FROM `attrib_storage` WHERE attributes_id='$id' AND publication_id='$_publication_id'");
				$isset = $nr->numRows();
				if ($isset) {
					//UPDATE
					$res1 =& $this->db->execute($sth2, array($_SESSION['feedback_modify'][$name], $id));
				} else {
					//INSERT
					$res1 =& $this->db->execute($sth1, array($id, $_SESSION['feedback_modify'][$name]));
				}
				if (DB::isError($res1)) {
					$this->db->rollBack();
					throw new SAException($res1->getMessage());
				}
			}
		}
		//delete form `attrib_storage` - if there is an attributte in database but its field in form is empty, simply delete it
		$empty_public_attr = $this->empty_ext_attr();
		$res =& $this->db->query("SELECT `attributes_id` FROM `attrib_storage` WHERE publication_id='$_publication_id'");
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		while ($row =& $res->fetchRow()) {
			$attributes_id = $row[0];
			if ($empty_public_attr[$attributes_id]) {
				$dr =& $this->db->query("DELETE FROM `attrib_storage` WHERE attributes_id='$attributes_id' AND publication_id='$_publication_id'");
				if (PEAR::isError($dr)) {
					$this->db->rollBack();
					throw new SAException($dr->getMessage());
				}
			}
		}
		$this->db->commit();
	}


	/**
	 * Recognize which extensible attribute have been assigned
	 *
	 * @return array of $attribute_id => $attrinute_name
	 */
	protected function filled_ext_attr() {
		foreach ($_SESSION['feedback_modify'] as $name => $value) {
			$sth = $this->db->prepare("SELECT id, name FROM `attributes` WHERE name=? AND confirmed='1'");
			if (PEAR::isError($sth)) {
				throw new SAException($sth->getMessage());
			}
			$res =& $this->db->execute($sth, $name);
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$id = $row[0]; $nm = $row[1];
				if ($_SESSION['feedback_modify'][$nm] != '') //if value of the extensible attribute in form stage 2 has been assigned
					$public_attr[$id] = $nm;
			}
		}
		return $public_attr;
	}

	/**
	 * Recognize which extensible attribute have not been assigned
	 *
	 * @return array of $attribute_id => $attrinute_name
	 */
	protected function empty_ext_attr() {
		foreach ($_SESSION['feedback_modify'] as $name => $value) {
			$sth = $this->db->prepare("SELECT id, name FROM `attributes` WHERE name=? AND confirmed='1'");
			if (PEAR::isError($sth)) {
				throw new SAException($sth->getMessage());
			}
			$res =& $this->db->execute($sth, $name);
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$id = $row[0]; $nm = $row[1];
				if ($_SESSION['feedback_modify'][$nm] == '') //if value of the extensible attribute in form stage 2 hasn't been assigned
					$public_attr[$id] = $nm;
			}
		}
		return $public_attr;
	}

	/**
	 * Return ids of categories assigned to the specified publication
	 *
	 * @param int $publication_id
	 * @return array array of category ids
	 */
	protected function assigned_categories($publication_id) {
		$sql = "SELECT categories_id
				FROM `categories_has_publication`
				WHERE publication_id = '$publication_id'";
		$data =& $this->db->getAll($sql, array(), DB_FETCHMODE_ORDERED);
		if(PEAR::isError($data)) throw new Exception($data->getMessage());
		foreach ($data as $i => $v) $data[$i] = $v[0];
		return $data;
	}
}
?>