<?php
require_once('Admin.inc.php');

final class AdminT4 extends Admin {

	/**
	 * Execute admin tab 4
	 *
	 */
	public function execute() {
		if ($this->post['add_category']) {
			try {
				$this->add_category($this->post['new_category_name']);
				$this->smarty->assign('operation_success', 'Kategorie byla přidána');
				$this->display_categories_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_categories_list();
			}
		} elseif ($ea=$this->post['edit_category']) {//$ea is array with only one element in format: $categories_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $categories_id => $v) {
					$this->display_edit_category($categories_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_category']) {
			try {
				$this->save_category($this->post['categories_id'], $this->post['name']);
				$this->smarty->assign('operation_success', 'Změny byly uloženy.');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=4');
				$this->smarty->display('admin_tab4.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_category($this->post['categories_id']);
			}
		} elseif ($da=$this->post['delete_category']) {//$da is array with only one element in format: ? => $categories_id
			try {
				if(is_array($da)) foreach($da as $categories_id => $v) {
					$this->delete_category($categories_id);
					$this->smarty->assign('operation_success', 'Kategorie byla smazána');
					$this->display_categories_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} elseif ($this->post['confirm_categories']) {
			try {
				$this->confirm_categories();
				$this->smarty->assign('operation_success', 'Všechny kategorie byly schváleny.');
				$this->display_categories_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());

			}
		} else {
			$this->display_categories_list();
		}
	}

	/**
	 * Show list of categories
	 *
	 */
	private function display_categories_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY confirmed"; break;
			case(2): $order_by = "ORDER BY name"; break;
			default: $order_by = "";
		}
		$sql = "SELECT DISTINCT id, name, confirmed
				FROM `categories` ".$order_by;
		$sql_sec = "
				SELECT categories_id
				FROM `categories_has_publication`
				GROUP BY categories_id
				HAVING COUNT(categories_id) > 0";
		$categories = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($categories)) foreach ($categories as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$categories[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_categories', 1);
		$this->smarty->assign('categories', $categories);
		$this->smarty->display('admin_tab4.tpl');
	}

	/**
	 * Display edit form of the category
	 *
	 * @param int $categories_id
	 */
	private function display_edit_category($categories_id) {
		$sql = "SELECT DISTINCT id, name
				FROM `categories`
				WHERE id='$categories_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_category', 1); //determine which part of template to use
		$this->smarty->assign('categories_id', $categories_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->display('admin_tab4.tpl');
	}

	/**
	 * Update name and description of any category identified by id
	 *
	 * @param int $categories_id
	 * @param string $new_name
	 */
	private function save_category($categories_id, $new_name) {
		if (sizeof($new_name) > 40)
			throw new Exception('Max. délka názvu kategorie je 40 znaků');
		if ($this->db->getOne("SELECT id FROM `categories` WHERE name='$new_name' AND id<>'$categories_id'"))
			throw new Exception('Kategorie se jménem '.$new_name.' již existuje');
		$res =& $this->db->query("UPDATE `categories` SET name='$new_name' WHERE id='$categories_id'");
		if (PEAR::isError($res)) {
    		throw new SAException($res->getMessage());
		}
	}

	/**
	 * Delete category
	 *
	 * @param int $categories_id
	 */
	private function delete_category($categories_id) {
		$this->db->autoCommit(false);
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		$res =& $this->db->query("DELETE FROM `categories_has_publication` WHERE categories_id='$categories_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage);
		}*/
		$res =& $this->db->query("DELETE FROM `categories` WHERE id='$categories_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}

	/**
	 * Set all categories as confirmed
	 *
	 */
	private function confirm_categories() {
		$res = $this->db->query("UPDATE `categories` SET confirmed=1 WHERE confirmed=0");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
	}

	/**
	 * Add new category into the database
	 *
	 * @param string $name
	 */
	private function add_category($name) {
		if (sizeof($name) > 40)
			throw new Exception('Max. délka názvu kategorie je 40 znaků');
		$submitter_id = $this->auth_user_id;
		//is there the same category in the database?
		if ($this->db->getOne("SELECT id FROM `categories` WHERE name='$name'"))
			throw new Exception('Kategorie se jménem '.$name.' již existuje');
		$sql = "INSERT INTO `categories` VALUES ('', '$submitter_id', '$name', 1)";
		$res =& $this->db->query($sql);
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage());
		}
	}
}
?>