<?php
require_once('Admin.inc.php');

/**
 * Managing publishers
 *
 */
final class AdminT6 extends Admin {

	/**
	 * Execute admin tab 6
	 *
	 */
	public function execute() {
		if ($this->post['add_publisher']) {
			try {
				$this->add_publisher($this->post['name']);
				$this->smarty->assign('operation_success', 'Vydavatel byl přidán');
				$this->display_publishers_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_publishers_list();
			}
		} elseif ($ea=$this->post['edit_publisher']) {//$ea is array with only one element in format: $publisher_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $publisher_id => $v) {
					$this->display_edit_publisher($publisher_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_publisher']) {
			try {
				$this->save_publisher($this->post['publisher_id'], $this->post['name']);
				$this->smarty->assign('operation_success', 'Změny byly uloženy.');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=6');
				$this->smarty->display('admin_tab6.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_publisher($this->post['publisher_id']);
			}
		} elseif ($da=$this->post['delete_publisher']) {//$da is array with only one element in format: [...] => $publisher_id
			try {
				if(is_array($da)) foreach($da as $publisher_id => $v) {
					$this->delete_publisher($publisher_id);
					$this->smarty->assign('operation_success', 'Vydavatel byl smazán');
					$this->display_publishers_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} elseif ($this->post['confirm_publishers']) {
			try {
				$this->confirm_publishers();
				$this->smarty->assign('operation_success', 'Všichni vydavatelé byli schváleni');
				$this->display_publishers_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());

			}
		} else {
			$this->display_publishers_list();
		}
	}

	/**
	 * Show list of publishers
	 *
	 */
	private function display_publishers_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY confirmed"; break;
			case(2): $order_by = "ORDER BY name"; break;
			default: $order_by = "";
		}
		$sql = "SELECT *
				FROM `publisher` ".$order_by;
		$sql_sec = "
				SELECT publisher_id
				FROM `publication`
				GROUP BY publisher_id
				HAVING COUNT(publisher_id) > 0";
		$publisher = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($publisher)) foreach ($publisher as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$publisher[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_publisher', 1);
		$this->smarty->assign('publisher', $publisher);
		$this->smarty->display('admin_tab6.tpl');
	}

	/**
	 * Display edit form of the publisher
	 *
	 * @param int $publisher_id
	 */
	private function display_edit_publisher($publisher_id) {
		$sql = "SELECT *
				FROM `publisher`
				WHERE id='$publisher_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_publisher', 1); //determine which part of template to use
		$this->smarty->assign('publisher_id', $publisher_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->display('admin_tab6.tpl');
	}

	/**
	 * Validate data sent with the web form
	 *
	 * @param string $name
	 */
	private function validate_publisher($name) {
		if (sizeof($name) > 45 || !$name) throw new Exception('Délka jména vydavatele je min. 1 a max. 45 znaků');
	}

	/**
	 * Update name and description of any publisher identified by id
	 *
	 * @param int $publisher_id
	 * @param string $name
	 */
	private function save_publisher($publisher_id, $name) {
		$this->validate_publisher($name);
		if ($this->db->getOne("SELECT id FROM `publisher` WHERE name='$name' AND id<>'$publisher_id'"))
			throw new Exception('Vydavatel se jménem '.$name.' již existuje');
		$res =& $this->db->query("UPDATE `publisher` SET name='$name' WHERE id='$publisher_id'");
		if (PEAR::isError($res)) {
    		throw new SAException($res->getMessage());
		}
	}

	/**
	 * Delete publisher
	 *
	 * @param int $publisher_id
	 */
	private function delete_publisher($publisher_id) {
		$this->db->autoCommit(false);
		$res =& $this->db->query("DELETE FROM `publisher` WHERE id='$publisher_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		//we must perform SET NULL manually, the refference definition in database is disabled because MySQL refference definition in this case behave strange
		$res =& $this->db->query("UPDATE `publication` SET publisher_id=0 WHERE publisher_id='$publisher_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}

	/**
	 * Set all publishers as confirmed
	 *
	 */
	private function confirm_publishers() {
		$res = $this->db->query("UPDATE `publisher` SET confirmed=1 WHERE confirmed=0");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
	}

	/**
	 * Add new publisher into the database
	 *
	 * @param string $name
	 */
	private function add_publisher($name) {
		$this->validate_publisher($name);
		$submitter_id = $this->auth_user_id;
		if ($this->db->getOne("SELECT id FROM `publisher` WHERE name='$name'"))
			throw new Exception('Vydavatel se jménem '.$name.' již existuje');
		$sql = "INSERT INTO `publisher` VALUES ('', '$submitter_id', '$name', 1)";
		$res =& $this->db->query($sql);
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage());
		}
	}
}
?>