<?php


/**
 * declares administration interface basic class
 *
 */
abstract class Admin
{
	protected $post;
	protected $get;
	protected $admin_uid;
	protected $db;
	public $smarty;
	protected $auth_module;
	protected $strict_mode;

	/*	Uses sessions:
		$_SESSION['feedback_modify'] - saved form data of admin_tab12.tpl template of section modify
		$_SESSION['tmp_attr'] - new extra attributes temporarily added by admin
		$_SESSION['attributes_deleted'] - extra ettributes flag
		$_SESSION['query_string'] - last query string for purposes of tab 1 - modify publication
		$_SESSION['selected_authors'] - authors choosen in  tab 1 - modify publication. This array is automatically deleted when clicking to another tab
	*/

	/**
	 * Factory method - takes number of the tab and creates appropriate successor class
	 * containing logic of a particular tab page.
	 *
	 * @param integer $tab_no number of the tab (1 - 7)
	 */
	public static function create($tab_no)
	{
		$post = clean_rec($_POST);
		$get = clean_rec($_GET);
		switch($tab_no) {
			//unconfirmed publications
			case('1'): $admin = new AdminT12($post, $get, 1, true); break;
			//confirmed publications
			case('2'): $admin = new AdminT12($post, $get, 2, true); break;
			//attributes
			case('3'): $admin = new AdminT3($post, $get); break;
			//categories
			case('4'): $admin = new AdminT4($post, $get); break;
			//authors
			case('5'): $admin = new AdminT5($post, $get); break;
			//publishers
			case('6'): $admin = new AdminT6($post, $get); break;
			//journals
			case('7'): $admin = new AdminT7($post, $get); break;
			//conferences
			case('8'): $admin = new AdminT8($post, $get); break;
			//templates
			case('9'): $admin = new AdminT9($post, $get); break;
			//users
			case('10'): $admin = new AdminT10($post, $get); break;
			default: $admin = new AdminT12($post, $get, 1, true); break;
		}
		return $admin;
	}

	/**
	 * Constructor
	 *
	 * @param string $post  $_POST input
	 * @param string $get  $_GET input
	 * @param boolean $needs_admin_rights
	 */
	public function __construct($post, $get, $needs_admin_rights=true)
	{
		$this->post = $post;
		$this->get = $get;
		$this->db = db_connect();
		$this->auth_module = new AuthModule($this->db);

		$this->smarty = new Smarty();
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;

		global $account_url;
		$this->smarty->assign('account_url', $account_url);
		global $app_path;
		$this->smarty->assign('app_path', $app_path);
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		//if no user logged in, force authorization
		if (!is_numeric($this->auth_module->get_id()))
		{
			$this->smarty->assign('refresh', "1 ; URL=../account/index.php?ref=/admin/");
			$this->smarty->assign('operation_error', 'To admin interface you must <a href="../account.php?ref=/admin/">log in</a>.');
			$this->smarty->display('admin_login.tpl');
			die();
		}
		elseif ($needs_admin_rights && !$this->auth_module->is_admin())
		{
			$ref = $_SERVER['HTTP_REFERER'];
			$this->smarty->assign('refresh', "1 ; URL=$ref");
			$this->smarty->assign('operation_error', "You don't have admin rights.");
			$this->smarty->display('admin_login.tpl');
			die();
		}
		$this->smarty->assign('level', $this->auth_module->get_level());
	}

	/**
	 * Execute admin
	 *
	 */
	abstract public function execute();

}

?>