<?php
require_once('Admin.inc.php');

/**
 * Managing publishers
 *
 */
final class AdminT6 extends Admin {

	/**
	 * Execute admin tab 6
	 *
	 */
	public function execute() {
		if ($this->post['add_publisher']) {
			try {
				add_publisher($this->db, $this->post['name'], $this->post['address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been added');
				$this->display_publishers_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_publishers_list();
			}
		} elseif ($ea=$this->post['edit_publisher']) {//$ea is array with only one element in format: $publisher_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $publisher_id => $v) {
					$this->display_edit_publisher($publisher_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_publisher']) {
			try {
				update_publisher($this->db, $this->post['publisher_id'], $this->post['name'], $this->post['address'], $this->auth_module->get_id(), true);
				$this->smarty->assign('operation_success', 'Publisher has been updated');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=6');
				$this->smarty->display('admin_tab6.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_publisher($this->post['publisher_id']);
			}
		} elseif ($da=$this->post['delete_publisher']) {//$da is array with only one element in format: [...] => $publisher_id
			try {
				if(is_array($da)) foreach($da as $publisher_id => $v) {
					$this->delete_publisher($publisher_id);
					$this->smarty->assign('operation_success', 'Publisher has been deleted');
					$this->display_publishers_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} else {
			$this->display_publishers_list();
		}
	}

	/**
	 * Show list of publishers
	 *
	 */
	private function display_publishers_list() {
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY address"; break;
			default: $order_by = "ORDER BY name";
		}
		$sql = "SELECT *
				FROM `publisher` ".$order_by;
		$sql_sec = "
				SELECT publisher_id
				FROM `publication`
				GROUP BY publisher_id
				HAVING COUNT(publisher_id) > 0";
		$publisher = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($publisher)) foreach ($publisher as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$publisher[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_publisher', 1);
		$this->smarty->assign('publisher', $publisher);
		$this->smarty->display('admin_tab6.tpl');
	}

	/**
	 * Display edit form of the publisher
	 *
	 * @param int $publisher_id
	 */
	private function display_edit_publisher($publisher_id) {
		$sql = "SELECT *
				FROM `publisher`
				WHERE id='$publisher_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_publisher', 1); //determine which part of template to use
		$this->smarty->assign('publisher_id', $publisher_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('address', $data['address']);
		$this->smarty->display('admin_tab6.tpl');
	}

	/**
	 * Delete publisher without fk checks
	 *
	 * @param int $publisher_id
	 */
	private function delete_publisher($publisher_id)
	{
		$this->db->autoCommit(false);
		$res =& $this->db->query("DELETE FROM `publisher` WHERE id='$publisher_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		//we must perform SET NULL manually, the refference definition in database is disabled because MySQL refference definition in this case behave strange
		$res =& $this->db->query("UPDATE `publication` SET publisher_id=0 WHERE publisher_id='$publisher_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}
}
?>