<?php
require_once('Admin.inc.php');

/**
 * Managing journals
 *
 */
final class AdminT7 extends Admin
{

	/**
	 * Execute admin tab 7
	 *
	 */
	public function execute()
	{
		if ($this->post['add_journal']) {
			try {
				add_journal($this->db, $this->post['name'], $this->post['issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been added');
				$this->display_journals_list();
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_journals_list();
			}
		} elseif ($ea=$this->post['edit_journal']) {//$ea is array with only one element in format: $journal_id => "změnit"
			try {
				if(is_array($ea)) foreach($ea as $journal_id => $v) {
					$this->display_edit_journal($journal_id);
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['save_edited_journal']) {
			try {
				update_journal($this->db, $this->post['journal_id'], $this->post['name'], $this->post['issn'], $this->auth_module->get_id(), true);
				$this->smarty->assign('operation_success', 'Journal has been updated');
				$this->smarty->assign('refresh', '1 ; URL=index.php?tab=7');
				$this->smarty->display('admin_tab7.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_edit_journal($this->post['journal_id']);
			}
		} elseif ($da=$this->post['delete_journal']) {//$da is array with only one element in format: [...] => $journal_id
			try {
				if(is_array($da)) foreach($da as $journal_id => $v) {
					$this->delete_journal($journal_id);
					$this->smarty->assign('operation_success', 'Journal has been deleted.');
					$this->display_journals_list();
				}
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
			}
		} else {
			$this->display_journals_list();
		}
	}

	/**
	 * Show list of journals
	 *
	 */
	private function display_journals_list()
	{
		switch($this->get['order']) {
			case(1): $order_by = "ORDER BY name"; break;
			case(2): $order_by = "ORDER BY issn"; break;
			default: $order_by = "ORDER BY name";
		}
		$sql = "SELECT *
				FROM `journal` ".$order_by;
		$sql_sec = "
				SELECT journal_id
				FROM `publication`
				GROUP BY journal_id
				HAVING COUNT(journal_id) > 0";
		$journal = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$warning = $this->db->getAll($sql_sec, array(), DB_FETCHMODE_ORDERED);
		if (is_array($journal)) foreach ($journal as $i => $v) { //assign warnings
			if (in_array(array(0 => $v['id']), $warning))
				$journal[$i]['warning'] = 1;
		}
		$this->smarty->assign('show_journal', 1);
		$this->smarty->assign('journal', $journal);
		$this->smarty->display('admin_tab7.tpl');
	}

	/**
	 * Display edit form of the journal
	 *
	 * @param int $journal_id
	 */
	private function display_edit_journal($journal_id)
	{
		$sql = "SELECT *
				FROM `journal`
				WHERE id='$journal_id'";
		$data =& $this->db->getRow($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($data)) {
    		throw new SAException($data->getMessage());
		}
		$this->smarty->assign('edit_journal', 1); //determine which part of template to use
		$this->smarty->assign('journal_id', $journal_id);
		$this->smarty->assign('name', $data['name']);
		$this->smarty->assign('issn', $data['issn']);
		$this->smarty->display('admin_tab7.tpl');
	}

	/**
	 * Delete publisher without fk checks
	 *
	 * @param int $publisher_id
	 */
	private function delete_journal($journal_id)
	{
		$this->db->autoCommit(false);
		$res =& $this->db->query("DELETE FROM `journal` WHERE id='$journal_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		//we must perform SET NULL manually, the refference definition in database is disabled because MySQL refference definition in this case behave strange
		$res =& $this->db->query("UPDATE `publication` SET journal_id=0 WHERE journal_id='$journal_id'");
		if(PEAR::isError($res)) {
			throw new SAException($res->getMessage);
		}
		$this->db->commit();
	}
}
?>